@@ -96,7 +96,7 @@
 #' \item{residuals}{a vector of the residuals (fitted vs measured)}
 #' \item{phases}{a dataframe of the phases used to produce the fitted pattern}
 #' \item{phases_grouped}{the phases dataframe grouped by phase_name and summed}
-#' \item{rwp}{the Rwp of the fitted vs measured pattern}
+#' \item{obj}{named vector of the objective parameters summarising the quality of the fit}
 #' \item{weighted_pure_patterns}{a dataframe of reference patterns used to produce the fitted pattern.
 #' All patterns have been weighted according to the coefficients used in the fit}
 #' \item{coefficients}{a named vector of coefficients used to produce the fitted pattern}
@@ -247,7 +247,7 @@ fps <- function(lib, ...) {
 #' \item{residuals}{a vector of the residuals (fitted vs measured)}
 #' \item{phases}{a dataframe of the phases used to produce the fitted pattern and their concentrations}
 #' \item{phases_grouped}{the phases dataframe grouped by phase_name and concentrations summed}
-#' \item{rwp}{the Rwp of the fitted vs measured pattern}
+#' \item{obj}{named vector of the objective parameters summarising the quality of the fit}
 #' \item{weighted_pure_patterns}{a dataframe of reference patterns used to produce the fitted pattern.
 #' All patterns have been weighted according to the coefficients used in the fit}
 #' \item{coefficients}{a named vector of coefficients used to produce the fitted pattern}
@@ -358,15 +358,6 @@ fps.powdRlib <- function(lib, smpl, harmonise, solver, obj, refs, std, force, st
 
   }
 
-
-#Make sure there aren't any negative counts
-  if (min(smpl[[2]]) < 0) {
-
-    stop("Please make sure that there are no negative count intensities in the sample data",
-         call. = FALSE)
-
-  }
-
 #Define force if missing
   if (missing(force)) {
 
@@ -559,6 +550,16 @@ fps.powdRlib <- function(lib, smpl, harmonise, solver, obj, refs, std, force, st
 
   }
 
+  #Make sure that Rwp isn't used if there are any negative counts
+  if (min(smpl[[2]]) < 0 & obj == "Rwp") {
+
+    cat("\n-Rwp could not be used as the objective function because there were negative
+         values in the counts data. Switched to the use of R as the objective function.")
+
+    obj = "R"
+
+  }
+
   #If the solver is "NNLS" and shift > 0, then don't shift
   if (solver == "NNLS" & shift > 0) {
 
@@ -690,12 +691,15 @@ lib$tth <- smpl[, 1]
 #-----------------------------------------------------------
 
 #The optimisation can fail if negative have creeped in during interpolation
-if(length(which(smpl[[2]] < 0) > 0)) {
+if(length(which(smpl[[2]] < 0) > 0) & obj == "Rwp") {
 
-  delete_negs <- which(smpl[[2]] < 0)
-  smpl <- smpl[-delete_negs,]
-  lib$tth <- lib$tth[-delete_negs]
-  lib$xrd <- lib$xrd[-delete_negs, ]
+  cat("Negative values present in interpolated data. Switching objective
+          function to R instead of Rwp to avoid errors.")
+  obj <- "R"
+  #delete_negs <- which(smpl[[2]] < 0)
+  #smpl <- smpl[-delete_negs,]
+  #lib$tth <- lib$tth[-delete_negs]
+  #lib$xrd <- lib$xrd[-delete_negs, ]
 
 }
 
@@ -782,12 +786,15 @@ if(shift > 0 & length(x) > 1 & solver %in% c("Nelder-Mead", "BFGS", "CG")) {
     cat("\n-Reoptimising after shifting data")
 
   #The optimisation can fail if negative have creeped in during interpolation
-  if(length(which(smpl[[2]] < 0) > 0)) {
+  if(length(which(smpl[[2]] < 0) > 0) & obj == "Rwp") {
 
-    delete_negs <- which(smpl[[2]] < 0)
-    smpl <- smpl[-delete_negs,]
-    lib$tth <- lib$tth[-delete_negs]
-    lib$xrd <- lib$xrd[-delete_negs, ]
+    cat("Negative values present in interpolated data. Switching objective
+         function to R instead of Rwp to avoid errors.")
+    obj <- "R"
+    #delete_negs <- which(smpl[[2]] < 0)
+    #smpl <- smpl[-delete_negs,]
+    #lib$tth <- lib$tth[-delete_negs]
+    #lib$xrd <- lib$xrd[-delete_negs, ]
 
   }
 
@@ -867,8 +874,20 @@ if (is.na(std_conc) | identical(names(x), std)) {
 df <- min_concs[[1]]
 dfs <- min_concs[[2]]
 
-#### Compute the Rwp
-R_fit <- sqrt(sum((1/smpl[,2]) * ((smpl[,2] - fitted_pattern)^2)) / sum((1/smpl[,2]) * (smpl[,2]^2)))
+#Objective parameters for results
+if (min(smpl[[2]]) < 0) {
+
+  Rwp_fit <- NA
+
+} else {
+
+  Rwp_fit <- sqrt(sum((1/smpl[[2]]) * ((smpl[[2]] - fitted_pattern)^2)) / sum((1/smpl[[2]]) * (smpl[[2]]^2)))
+
+}
+
+R_fit <- sqrt(sum((smpl[[2]] - fitted_pattern)^2)/sum(smpl[[2]]^2))
+
+delta_fit <- sum(abs(smpl[[2]] - fitted_pattern))
 
 #Extract the xrd data
 xrd <- data.frame(lib$xrd,
@@ -908,7 +927,9 @@ out <- list("tth" = smpl[,1],
             "residuals" = unname(resid_x),
             "phases" = df,
             "phases_grouped" = dfs,
-            "rwp" = R_fit,
+            "obj" = c("Rwp" = Rwp_fit,
+                      "R" = R_fit,
+                      "Delta" = delta_fit),
             "weighted_pure_patterns" = xrd,
             "coefficients" = x,
             "inputs" = inputs)
